import React, { useState } from "react";
import {
  PRE_GAME,
  PRE_ROUND,
  IN_ROUND,
  POST_ROUND,
  POST_GAME,
  ERROR,
} from "./GameState";
import HeaderAndFooter from "./HeaderAndFooter";
import PreGame from '../screens/PreGame';
import PreRound from '../screens/PreRound';
import GamePanel from "../screens/GamePanel";
import RoundSummary from '../screens/RoundSummary';
import PlayerScores from "../screens/PlayerScores";
import { gameIdStore } from "../../domain/store";

const initialState = {
  gameState: PRE_GAME,
  lastRound: null,
  joined: false,
}

const extractAndRemoveSearchParam = param => {
  const u = new URL(window.location.href);
  const extracted = u.searchParams.get(param);
  u.searchParams.delete(param);
  window.history.replaceState({}, document.title, u.href);
  return extracted;
}

const Game = () => {
  const [ state, rawSetState ] = useState(initialState);

  const setGameState = gameState => rawSetState({ ...state, gameState });
  const setGameStateAnd = (gameState, updates) => rawSetState({ ...state, gameState, ...updates });
  const onGameJoined = () => setGameStateAnd(PRE_ROUND, { joined: true });
  
  const joinCode = extractAndRemoveSearchParam("join");
  if (joinCode) {
    gameIdStore.set(joinCode);
    setGameState(PRE_ROUND);
  }

  const summaryCode = extractAndRemoveSearchParam("summary");
  if (summaryCode) {
    gameIdStore.set(summaryCode);
    setGameState(POST_GAME);
  }

  switch (state.gameState) {
    case PRE_GAME:
      return (
        <HeaderAndFooter>
          <PreGame
            onGameJoined={onGameJoined}
          />
        </HeaderAndFooter>
      );
    case PRE_ROUND:
      return (
        <HeaderAndFooter>
          <PreRound
            joined={state.joined}
            onGameJoined={onGameJoined}
            onStart={() => setGameState(IN_ROUND)}
          />
        </HeaderAndFooter>
      );
    case IN_ROUND:
      return <GamePanel
        onRoundEnd={lastRound => setGameStateAnd(POST_ROUND, { lastRound })}
        onGameEnd={() => setGameState(POST_GAME)}
      />
    case POST_ROUND:
      return <RoundSummary
        round={state.lastRound}
        onNext={() => setGameState(IN_ROUND)}
      />
    case POST_GAME:
      return (
        <HeaderAndFooter>
          <PlayerScores
            onReturnToStart={() => {
              gameIdStore.set(null);
              setGameStateAnd(PRE_GAME, { joined: false });
            }}
          />
        </HeaderAndFooter>
      );
    case ERROR:
      // TODO - would be nice to hook this into the sub-components, maybe with a HOC?
      return <p>Application encountered unrecoverable error, please refresh the page.</p>
    default:
      setGameState(ERROR);
      return <p>Application state is inconsistent, please refresh and rejoin your previous game.</p>
  }
}

export default Game;
